This program implements the multi-label classification using error-correcting
codes (ML-ECC) framework in the paper "Multi-label Classification Using Error-
correcting Codes of Hard or Soft Bits" by Ferng and Lin, 2013.

Copyright information can be found in the file COPYRIGHT.

If there are any questions, please feel free to contact the authors at
r99922054@csie.ntu.edu.tw Chun-Sung Ferng, or
htlin@csie.ntu.edu.tw Hsuan-Tien Lin.

Set up
======

Prerequisite
------------

  * g++ (tested on 4.7.2)
  * python (tested on 2.7.3)
  * LibSVM (for svm learner, tested on 3.0)
  * LibLINEAR (for linear learner, tested on 1.91)
  * Weka & Mulan (for rf learner, tested on Weka 3.7.1 and Mulan 1.2.0)
  * MATLAB with statistics toolbox (for gmm learner, tested on R2012a)

Configuration
-------------

  * Set the corresponding path to external library in `learner/Makefile`
  * Make sure that `java` and `matlab` are on your path
  * Execute `make` to build the programs

Usage
=====

mlecc.py train [-h] [--code {rrep,hamr,bch,ldpc,clr,none}] [--length LENGTH]
	[--base BASE] [--learner {svm,rf,linear,gmm}] DATA LABELS MODEL-DIR

  * DATA: training data file, without extension (See Data Format Section)
  * LABELS: training label file
  * MODEL-DIR: directory for writing models and temp files

  * -h: print help message
  * --code: the error-correcting code to use
    * rrep: RAkEL repetition code
    * hamr: Hamming on repetition code
    * bch: BCH code
    * ldpc: Low-density parity-check code
  * --length: the desired code length (no effect when code=clr,none)
  * --base: 1 for Binary Relevance, k>1 for k-powerset
  * --learner: underlying binary/multi-class learner
    * svm: SVM with RBF Kernel
    * linear: Linear SVM
    * rf: Random Forests 
    * gmm: Gaussian Mixture Models

mlecc.py test [-h] [--decoder {alg-hard,geo-hard,geo-soft}] [--truth TRUTH]
	DATA MODEL-DIR LABELS-OUTPUT

  * DATA: testing data file, without extension (See Data Format Section)
  * MODEL-DIR: directory for reading models and writing temp files
  * LABELS-OUTPUT: output label file

  * -h: print help message
  * --decoder: the decoder to use
    * alg-hard: the off-the-shelf decoder
    * geo-hard: the geometric decoder with hard input
    * geo-soft: the geometric decoder with soft input
  * --truth: ground truth label file (will do evaluation if this present)

Example
-------
./mlecc.py train --code hamr --length 127 --base 3 --learner rf \
	sample/emotions-train sample/emotions-train.label PATH/TO/MODEL
./mlecc.py test --decoder geo-hard --truth sample/emotions-test.label \
	sample/emotions-test PATH/TO/MODEL PATH/TO/PREDICTION

Data Format
===========

DATA File
---------

The data format depends on the learner you would like to use. Assume
your `--data` parameter is `xD`. If your learner is

  * rf: the program will read `xD.arff` and `xD.xml` in Weka/Mulan format.
  * svm, linear: the program will read `xD.svm` in LibSVM format.
  * gmm: the program will read `xD.txt` as a plain text, dense matrix of
         features. Each line describes one instance, and the features are
	 separated by a whitespace.

Note that in any format the label descriptions in DATA will be discarded.
Please specify the label information in LABELS.

LABELS File
-----------

The LABELS file is a dense matrix of labels of instances. Each line corresponds
to one instance, and each character represents one label. The `j`-th character
on the `i`-th line is '1'  if the `j`-th label presents in the `i`-th instance,
or '0' otherwise. There is no whitespace separating the labels.

LABELS-OUTPUT File
------------------

The prediction output will be in the same format as LABELS when using algebraic
decoders. When using geometric decoder,instead of binary value for each label,
the predictions will be in real value. Also, the prediction of labels for one
instance will be separated by a whitespace.

Pre-specified Parameters of Underlying Learners
===============================================

You may change the parameters by modifying `learner/kps-*-train` and then
re-build the program. Other parameters not specified here take the default
value in the implementation.

  * svm: RBF kernel, probability estimate enabled, epsilon 1e-5, cache 8192M,
         doing 5-fold CV to choose C and gamma from {2^-5, 2^-3, ..., 2^7} 
	 and {2^-7, 2^-5, ..., 2^3} respectively.
  * linear: using L2-regularized logistic regression (primal) solver, epsilon
         1e-5, bias 1, doing 5-fold CV to choose C from {2^-1, 2^1, ..., 2^9}.
  * rf: 60 trees, no pruning.
  * gmm: choosing number of components from {1,2,4,8,16} by BIC, using diagonal
         covariance matrices with 1e-7 regularization.
