#include <assert.h>
#include <math.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <vector>
using namespace std;

#include "solver.h"
#include "util.h"

enum LossFunc {L0, L1, L2, L1OSR};

int L, N;
char *mdl, *lbl, *bit, *logs = NULL;
int I = 10, R = 20, VERBOSE = -1;
LossFunc loss = L2;

inline void parseArgs(int argc, char *argv[]) {
	for(int i = 1; i < argc; ++i) {
		if(strcmp(argv[i], "-model") == 0 && i+1 < argc) {
			++i;
			mdl = argv[i];
		} else if(strcmp(argv[i], "-label") == 0 && i+1 < argc) {
			++i;
			lbl = argv[i];
		} else if(strcmp(argv[i], "-bits") == 0 && i+1 < argc) {
			++i;
			bit = argv[i];
		} else if(strcmp(argv[i], "-log") == 0 && i+1 < argc) {
			++i;
			logs = argv[i];
		} else if(strcmp(argv[i], "-verbose") == 0 && i+1 < argc) {
			++i;
			sscanf(argv[i], "%d", &VERBOSE);
		} else if(strncmp(argv[i], "L=", 2) == 0) {
			if(strcmp(argv[i]+2, "l0") == 0) loss = L0;
			else if(strcmp(argv[i]+2, "l1") == 0) loss = L1;
			else if(strcmp(argv[i]+2, "l2") == 0) loss = L2;
			else if(strcmp(argv[i]+2, "l1osr") == 0) loss = L1OSR;
		} else if(strncmp(argv[i], "I=", 2) == 0) {
			sscanf(argv[i]+2, "%d", &I);
		} else if(strncmp(argv[i], "R=", 2) == 0) {
			sscanf(argv[i]+2, "%d", &R);
		}
	}
	assert(mdl != NULL && lbl != NULL && bit != NULL);
}

inline double dist(double *x, int *y) {
	double s = 0;
	for(int i = 0; i < 7; ++i)
		s += fabs(x[i]-y[i]);
	return s;
}

int main(int argc, char *argv[]) {
	setRandSeed();
	parseArgs(argc, argv);

	Solver *solver = NULL;
	FILE *fp = fopen(mdl, "r");
	fscanf(fp, "%d %d\n", &N, &L);
	if(loss == L2) solver = new SolverL2(N, L);
	else if(loss == L1) solver = new SolverL1(N, L);
	else if(loss == L1OSR) solver = new SolverL1Osr(N, L);
	VVI &label2code = solver->getLabel2Code();
	VVI &code2label = solver->getCode2Label();

	char *s;
	s = (char*)malloc(sizeof(char)*(N+3));
	for(int i = 0; i < L; ++i) {
		fgets(s, N+2, fp);
		for(int j = 0; j < N; ++j)
			if(s[j] == '1') {
				code2label[i].push_back(j);
				label2code[j].push_back(i);
			}
	}
	fclose(fp);

	char *s2 = (char*)malloc(sizeof(char)*(L*30+3));
	double *x = (double*)malloc(sizeof(double)*(N+3));
	double *val = (double*)malloc(sizeof(double)*(L+3));

	char *toSave = (char*)malloc(sizeof(char)*(I+3));
	memset(toSave, -1, sizeof(char)*(I+3));
	toSave[I] = 1;
	if(VERBOSE > 0) for(int i = 0; i < I; i += VERBOSE) toSave[i] = 1;
	double **xSav = (double**)malloc(sizeof(double*)*(I+3));
	FILE **flbl = (FILE**)malloc(sizeof(FILE*)*(I+3));
	FILE **flog = (logs==NULL) ? NULL : (FILE**)malloc(sizeof(FILE*)*(I+3));
	for(int i = 0; i <= I; ++i) {
		xSav[i] = (double*)malloc(sizeof(double)*(N+3));
		if(toSave[i] < 0) continue;
		if(VERBOSE < 0) {
			flbl[i] = fopen(lbl, "w");
			if(logs != NULL) flog[i] = fopen(logs, "w");
		} else {
			char path[4096];
			sprintf(path, "%s.%d", lbl, i);
			flbl[i] = fopen(path, "w");
			if(logs != NULL) {
				sprintf(path, "%s.%d", logs, i);
				flog[i] = fopen(path, "w");
			}
		}
	}

	FILE *fbit = fopen(bit, "r");
	double *loss = (double*)malloc(sizeof(double)*(I+3));
	while(fgets(s2, L*30+2, fbit) != NULL) {
		char *p = strtok(s2, " ");
		for(int i = 0; p != NULL && i < L; ++i, p = strtok(NULL, " ")) {
			sscanf(p, "%lg", &val[i]);
			val[i] = convertFrom01(val[i]);
		}

		for(int i = 0; i <= I; ++i) loss[i] = 1e100;
		for(int rt = 0; rt < R; ++rt) {
			solver->solveSetup(val, x);
			for(int i = 0; i <= I; ++i) {
				if(i > 0) solver->solveIter();
				if(solver->calculateLoss() < loss[i]) {
					loss[i] = solver->calculateLoss();
					memcpy(xSav[i], x, sizeof(double)*N);
				}
			}
			solver->solveFinal();
		}
		for(int i = 0; i <= I; ++i) {
			if(toSave[i] < 0) continue;
			for(int j = 0; j < N; ++j)
				fprintf(flbl[i], "%lf%c", convertTo01Real(xSav[i][j]), (j==N-1)?'\n':' ');
			if(flog != NULL && flog[i] != NULL) fprintf(flog[i], "%.12lg\n", loss[i]);
		}
	}
	fclose(fbit);
	for(int i = 0; i <= I; ++i) {
		if(toSave[i] < 0) continue;
		fclose(flbl[i]);
		if(flog != NULL) fclose(flog[i]);
	}
}

