import collections
import json
import math
import os
import random
import sys

import util

def countSame(a, p):
	return sum(map(lambda (x,y): 1 if x==y else 0, zip(a, p)))

def calcPrecision(a, p):
	ppos = sum(p)
	if ppos == 0: return 0 if sum(a) == 0 else 1
	else: return sum( x*y for (x,y) in zip(a,p) ) / float(ppos)

def calcRecall(a, p):
	apos = sum(a)
	if apos == 0: return 0 if sum(p) == 0 else 1
	else: return sum( x*y for (x,y) in zip(a,p)) / float(apos)

def calcF1(a, p):
	pc = calcPrecision(a, p)
	rc = calcRecall(a, p)
	return 0 if pc+rc==0 else 2*pc*rc/(pc+rc)

def calcMinPR(a, p):
	apos = sum(a)
	ppos = sum(p)
	return 1 if apos == 0 and ppos == 0 else sum( x*y for (x,y) in zip(a,p) ) / float(max(apos, ppos))

def calcJaccard(a, p):
	apos = sum(a)
	ppos = sum(p)
	x = sum( x*y for (x,y) in zip(a,p) )
	return 1 if apos == 0 and ppos == 0 else x/float(apos+ppos-x)

def distL1(a, p):
	return util.avg( [0 if x<0 else abs(x-y) for (x,y) in zip(a, p)] )

def calcRankLoss(a, p):
	p1 = map(lambda j: p[j], filter(lambda i: a[i]==1, range(len(a))))
	p0 = map(lambda j: p[j], filter(lambda i: a[i]==0, range(len(a))))
	return util.avg([ (cmp(y,x)+1)/2. for x in p1 for y in p0 ])

def calcAvgPrec(a, p):
	if sum(a) == 0: return 1
	ls = map(lambda (x,y): x, sorted(zip(a,p), key=lambda (x,y): -y))
	accu = reduce(lambda (a,l),x: (a+x,l+[a+x]), ls, (0,[]))[1]
	return sum( 0 if x==0 else y/float(i+1) for (i,(x,y)) in enumerate(zip(ls,accu)) ) / float(sum(a))

def calcAvgAndSte(xs):
	n = float(len(xs))
	s = sum(xs) / n
	s2 = sum([ x*x for x in xs ])
	return s, math.sqrt((s2-n*s*s)/(max(1.0,n-1.0)))/math.sqrt(n)

def average(lsResult):
	res = {"bitdist":{}, "bitacc":{}, "bithamp":{}, "log":{}}
	for ms in ["acc", "ham", "bitham", "microf1", "macrof1", "rankloss", "precision", "recall", "minpr", "jaccard", "avgprec"]:
		res["log"][ms] = [ r[ms] for r in lsResult ]
		res[ms], res[ms+"_ste"] = calcAvgAndSte(res["log"][ms])
	for r in lsResult:
		for d in r["bitcnt"].keys():
			res["bitdist"][d] = res["bitdist"].get(d, 0) + r["bitcnt"][d]
			res["bitacc"][d] = res["bitacc"].get(d, 0) + r["bitacc"][d]
			res["bithamp"][d] = res["bithamp"].get(d, 0) + r["bithamp"][d]
	s = float(sum(res["bitdist"].values()))
	for d in res["bitdist"].keys():
		res["bitacc"][d] /= float(res["bitdist"][d])
		res["bithamp"][d] /= float(res["bitdist"][d])
		res["bitdist"][d] /= s
	res["n"] = len(lsResult)
	
	if len(lsResult) > 0 and lsResult[0].has_key("objective"):
		res["objective"], res["objective_ste"] = calcAvgAndSte([ r["objective"] for r in lsResult ])
		res["objectiveC"], res["objectiveC_ste"] = calcAvgAndSte([ r["objectiveC"] for r in lsResult ])
	if len(lsResult) > 0 and lsResult[0].has_key("bitxor"):
		res["bitxor"] = collections.defaultdict(lambda: {"cnt":0, "l1":0, "one":0})
		for r in lsResult:
			for (k,v) in r["bitxor"].items():
				for k2 in ["cnt", "l1", "one"]:
					res["bitxor"][k][k2] += v[k2]
		for (k,v) in res["bitxor"].items():
			v["l1"] /= float(v["cnt"])
			v["one"] /= float(v["cnt"])
	return res

def save(result, filename):
	fp = open(filename, 'w')
	fp.write('{\n')
	fp.write(',\n'.join([ json.dumps(k)+': '+json.dumps(obj=result[k], sort_keys=True) for k in sorted(result.keys()) ])+'\n')
	fp.write('}\n')
	fp.flush()
	fp.close()

def avgAndSave(lsResult, filename):
	res = average(lsResult)
	save(res, filename)

def evalOne(aLabel, aCodeword, pCodeword, pLabel):
	util.logColor(["evaluate.evalOne", aLabel, str(aCodeword), str(pCodeword), pLabel], util.COLOR_WHITE)
	aLabel = map(lambda s: map(int, list(s)), util.readLines(aLabel))
	pLabelR = map(lambda s: map(float, s.split() if s.count(' ')>0 else list(s)), util.readLines(pLabel))
	pLabelB = [ [ int(x+0.5) if x!=0.5 else random.choice([0,1]) for x in a ] for a in pLabelR ]
	assert len(aLabel) == len(pLabelB), "label file not match %d %d"%(len(aLabel), len(pLabelB))

	r = {}
	r["acc"] = countSame(aLabel, pLabelB) / float(len(aLabel))
	r["ham"] = util.avg(map(lambda (a,p): 1.-countSame(a,p)/(len(a)+0.), zip(aLabel, pLabelB)))
	r["microf1"] = calcF1(sum(aLabel, []), sum(pLabelB, []))
	r["macrof1"] = util.avg(map(lambda i: calcF1(sum([[a[i]] for a in aLabel], []), sum([[p[i]] for p in pLabelB], [])), range(len(aLabel[0]))))
	r["rankloss"] = util.avg(map(lambda (a,p): calcRankLoss(a,p), zip(aLabel, pLabelR)))
	r["precision"] = util.avg(map(lambda (a,p): calcPrecision(a,p), zip(aLabel, pLabelB)))
	r["recall"] = util.avg(map(lambda (a,p): calcRecall(a,p), zip(aLabel, pLabelB)))
	r["minpr"] = util.avg(map(lambda (a,p): calcMinPR(a,p), zip(aLabel, pLabelB)))
	r["jaccard"] = util.avg(map(lambda (a,p): calcJaccard(a,p), zip(aLabel, pLabelB)))
	r["avgprec"] = util.avg(map(lambda (a,p): calcAvgPrec(a,p), zip(aLabel, pLabelR)))

	if aCodeword is not None and pCodeword is not None:
		aCodeword = [ [ -1 if c=='-' else int(c) for c in s ] for s in util.readLines(aCodeword)[1:] ]
		pCodewordR = map(lambda s: map(float, s.split() if s.count(' ')>0 else list(s)), util.readLines(pCodeword))
		pCodewordB = [ [ int(x+0.5) if x!=0.5 else random.choice([0,1]) for x in a ] for a in pCodewordR ]
		assert len(aCodeword) == len(pCodewordB), "codeword file not match"
		r["bitham"] = util.avg(map(lambda (a,p): distL1(a, p), zip(aCodeword, pCodewordR)))
		bitcnt = {}
		bitacc = {}
		bithamp = {}
		for (al, pl, ac, pc) in zip(aLabel, pLabelB, aCodeword, pCodewordR):
			d = distL1(ac, pc) * float(len(ac))
			d = d - math.fmod(d, 1)		# quantize
			bitcnt[d] = bitcnt.get(d, 0) + 1
			bitacc[d] = bitacc.get(d, 0) + (1 if al==pl else 0)
			bithamp[d] = bithamp.get(d, 0) + 1.0-countSame(al,pl)/(len(al)+0.0)
		r["bitcnt"] = bitcnt
		r["bitacc"] = bitacc
		r["bithamp"] = bithamp
	return r

