import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.ObjectInputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Random;
import java.util.Scanner;

import mulan.data.MultiLabelInstances;
import mulan.transformations.RemoveAllLabels;

import weka.classifiers.AbstractClassifier;
import weka.classifiers.Classifier;
import weka.core.Attribute;
import weka.core.Instances;

public class KpsWekaPredict {

	protected static int N;
	protected static int L;
	protected static String modelPath;
	protected static String bitPath;
	protected static double[][] bit;
	protected static Instances dataset;

	protected static ArrayList<String> genValueList(int n) {
		ArrayList<String> val = new ArrayList<String>();
		for(int i = 0; i < n; ++i)
			val.add(String.valueOf(i));
		return val;
	}

	protected static void readData(String path) throws Exception {
		MultiLabelInstances mldata = new MultiLabelInstances(path+".arff", path+".xml");
		dataset = RemoveAllLabels.transformInstances(mldata);
//		dataset = mldata.getDataSet();

//		int[] label = mldata.getLabelIndices();
//		Arrays.sort(label);
//		for(int i = label.length-1; i >= 0; --i)
//			dataset.deleteAttributeAt(label[i]);
	}

	protected static void parseArgs(String[] args) throws Exception {
		String dataPath = null;
		for(int i = 0; i < args.length; ++i) {
			if(i+1 < args.length && args[i].equals("-data")) {
				++i;
				dataPath = args[i];
			}
			else if(i+1 < args.length && args[i].equals("-model")) {
				++i;
				modelPath = args[i];
			}
			else if(i+1 < args.length && args[i].equals("-bit")) {
				++i;
				bitPath = args[i];
			}
		}
		assert bitPath != null;
		readData(dataPath);
	}

	protected static Classifier readLearner(String path) throws Exception {
		ObjectInputStream ois = new ObjectInputStream(new FileInputStream(path));
		Classifier learner = (Classifier)ois.readObject();
		ois.close();
		return learner;
	}

	public static void main(String[] args) throws Exception {
		parseArgs(args);

		Scanner scan = new Scanner(new BufferedReader(new FileReader(modelPath)));
		N = scan.nextInt();
		L = scan.nextInt();
		bit = new double[dataset.numInstances()][L];

		int m = scan.nextInt();
		Random rnd = new Random();
		for(int i = 0; i < m; ++i) {
			Classifier learner = readLearner(scan.next());
			int[] pos = new int[scan.nextInt()];
			for(int j = 0; j < pos.length; ++j)
				pos[j] = scan.nextInt();

			{
				int tmp = dataset.classIndex();
				Attribute cls = new Attribute("myClass", genValueList(1<<pos.length));
				int idx = dataset.numAttributes();
				dataset.insertAttributeAt(cls, idx);
				dataset.setClassIndex(idx);
				if(tmp >= 0) dataset.deleteAttributeAt(tmp);
			}

			for(int j = 0; j < dataset.numInstances(); ++j) {
				double[] dist = learner.distributionForInstance(dataset.get(j));
				/* hard decision
				int v = 0, nv = 1;
				for(int k = 1; k < dataset.numClasses(); ++k) {
					if(dist[k] > dist[v]) {
						v = k;
						nv = 1;
					}
					else if(dist[k] == dist[v]) {
						++nv;
						if(rnd.nextInt(nv) == 0) v = k;
					}
				}
				v = Integer.parseInt(dataset.classAttribute().value(v));
				for(int k = pos.length-1; k >= 0; --k, v >>= 1)
					bit[j][pos[k]] = ((v&1)==1) ? '1' : '0';
				*/
				for(int k = 1; k < dataset.numClasses(); ++k) {
					int v = Integer.parseInt(dataset.classAttribute().value(k));
					for(int p = pos.length-1; p >= 0; --p, v >>= 1)
						bit[j][pos[p]] += ((v&1)==1) ? dist[k] : 0;
				}
			}
		}
		scan.close();
		PrintStream fbit = new PrintStream(bitPath);
		for(int i = 0; i < bit.length; ++i) {
			//fbit.println(String.valueOf(bit[i]));
			fbit.printf("%f", bit[i][0]);
			for(int j = 1; j < bit[i].length; ++j)
				fbit.printf(" %f", bit[i][j]);
			fbit.println();
		}
		fbit.flush();
		fbit.close();
	}
}


