import java.io.BufferedReader;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.ObjectOutputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;

import mulan.data.MultiLabelInstances;
import mulan.transformations.RemoveAllLabels;

import weka.classifiers.AbstractClassifier;
import weka.classifiers.Classifier;
import weka.core.Attribute;
import weka.core.Instances;

public class KpsWekaTrain {

	protected static int K = 3;
	protected static int N;
	protected static int L;
	protected static String modelPath;
	protected static char[][] bit;
	protected static Instances dataset;
	protected static String learnerName;

	protected static ArrayList<String> genValueList(int n) {
		ArrayList<String> val = new ArrayList<String>();
		for(int i = 0; i < n; ++i)
			val.add(String.valueOf(i));
		return val;
	}

	protected static void readData(String path) throws Exception {
		MultiLabelInstances mldata = new MultiLabelInstances(path+".arff", path+".xml");
		dataset = RemoveAllLabels.transformInstances(mldata);
//		dataset = mldata.getDataSet();

//		int[] label = mldata.getLabelIndices();
//		Arrays.sort(label);
//		for(int i = label.length-1; i >= 0; --i)
//			dataset.deleteAttributeAt(label[i]);
	}

	protected static void readBits(String path) throws Exception {
		BufferedReader br = new BufferedReader(new FileReader(path));
		ArrayList<String> tmp = new ArrayList<String>();
		String s = br.readLine();
		String[] sp = s.split(" ");
		N = Integer.parseInt(sp[0]);
		L = Integer.parseInt(sp[1]);
		for(s = br.readLine(); s != null; s = br.readLine())
			tmp.add(s);
		bit = new char[tmp.size()][];
		for(int i = 0; i < bit.length; ++i)
			bit[i] = tmp.get(i).toCharArray();
		assert bit[0].length == L;
	}

	protected static String getBaseModelFileName(int id) {
		if(id < 10) return modelPath + "/base00" + id + ".mdl";
		else if(id < 100) return modelPath + "/base0" + id + ".mdl";
		else return modelPath + "/base" + id + ".mdl";
	}

	protected static void parseArgs(String[] args) throws Exception {
		String dataPath = null, bitPath = null;
		for(int i = 0; i < args.length; ++i) {
			if(i+1 < args.length && args[i].equals("-data")) {
				++i;
				dataPath = args[i];
				args[i] = args[i-1] = "";
			}
			else if(i+1 < args.length && args[i].equals("-K")) {
				args[i] = args[i].toLowerCase();
				++i;
				K = Integer.parseInt(args[i]);
				args[i] = args[i-1] = "";
			}
			else if(i+1 < args.length && args[i].equals("-model")) {
				++i;
				modelPath = args[i];
				args[i] = args[i-1] = "";
			}
			else if(i+1 < args.length && args[i].equals("-bit")) {
				++i;
				bitPath = args[i];
				args[i] = args[i-1] = "";
			}
			else if(i+1 < args.length && args[i].equals("-learner")) {
				++i;
				learnerName = args[i];
				args[i] = args[i-1] = "";
			}
		}
		assert K > 0;
		assert bitPath != null;
		readData(dataPath);
		readBits(bitPath);
	}

	public static void main(String[] args) throws Exception {
		parseArgs(args);

		int i, j, k, m;
		int seg[] = new int[(L+K-1)/K+1];
		for(i = j = 0; j < L; ++i, j += K) seg[i] = j;
		m = i;
		seg[m] = L;
		if(L%K == 1) seg[--m] = L;

		PrintStream fmdl = new PrintStream(modelPath+"/model.mdl");
		fmdl.printf("%d %d\n", N, L);
		fmdl.printf("%d\n", m);
		for(i = 0; seg[i] < L; ++i) {
			fmdl.printf("%s %d", getBaseModelFileName(i), seg[i+1]-seg[i]);
			for(j = seg[i]; j < seg[i+1]; ++j)
				fmdl.printf(" %d", j);
			fmdl.printf("\n");
		}
		fmdl.flush();
		fmdl.close();

		for(i = 0; seg[i] < L; ++i) {
			if(i == 0 || seg[i+1]-seg[i] != seg[i]-seg[i-1]) {
				int tmp = dataset.classIndex();
				Attribute cls = new Attribute("myClass", genValueList(1<<(seg[i+1]-seg[i])));
				int idx = dataset.numAttributes();
				dataset.insertAttributeAt(cls, idx);
				dataset.setClassIndex(idx);
				if(tmp >= 0)
					dataset.deleteAttributeAt(tmp);
			}

			for(j = 0; j < bit.length; ++j) {
				for(m = 0, k = seg[i]; k < seg[i+1]; ++k)
					m = (m<<1) | (bit[j][k]=='1'?1:0);
				dataset.get(j).setClassValue(dataset.classAttribute().value(m));
			}
			// train RF
			Classifier learner = AbstractClassifier.forName(learnerName, args);
			learner.buildClassifier(dataset);

			ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream(getBaseModelFileName(i)));
			oos.writeObject(learner);
			oos.close();
		}
	}
}

