function [mdl] = gmmTrain(X, Y)
	ys = [min(Y):max(Y)];
	if min(Y) == max(Y)
		mdl = struct('ys', ys);
		return
	end
	ys = ys(hist(Y, ys)>0);

	grpYX = [];
	for y = ys
		grpYX{y+1} = X(Y==y, :);
	end

%	reg = cvRegularize(grpYX, ys);
	reg = 1e-7;
	mdl = gmmFit(grpYX, ys, reg);
end

function [mdl] = gmmFit(grpYX, ys, reg)
	P_NCOMP = [1 2 4 8 16];
	gmms = [];
	prior = [];
	for y = ys
		x = grpYX{y+1};
		mbic = 1e100;
		r = [];
		for nc = P_NCOMP
			if size(x, 1) < nc
				continue
			end
			try
				t = gmmFitOne(x, nc, reg);
				bic = 1e99;
				if ~isempty(t.gmd.BIC)
					bic = t.gmd.BIC;
				end
				if isempty(r) | bic < mbic
					mbic = bic;
					r = t;
				end
			catch err
				rethrow(err)
			end
		end
		if size(r, 1) == 0
			error('No GMM fitted')
		end
		gmms{y+1} = r;
		prior{y+1} = size(x,1);
	end
	mdl = struct('ys', ys, 'gmms', gmms, 'prior', prior);
end

function [gmm] = gmmFitOne(X, ncomp, reg)
	varX = var(X);
	col = find(varX >= eps(max(varX))*size(X,1));	% const check in gmdistribution.fit
	%col = find(sum( X - repmat(X(1,:), size(X,1), 1) ));	% non-const columns
	%if size(col, 2) == 0
	%	d = size(X,2);
	%	gmd = gmdistribution(X(1,:), eye(d)*reg, [1.0]);
	%	col = [1:d];
	%	gmm = struct('col', col, 'gmd', gmd);
	if size(X,1) == 1
		gmd = gmdistribution(X, eye(size(X,2))*reg, [1.0]);
		gmm = struct('col', col, 'gmd', gmd);
	elseif isempty(col)
		d = size(X,2);
		gmd = gmdistribution(mean(X,1), diag(sqrt(varX)+reg), [1.0]);
		col = [1:d];
		gmm = struct('col', col, 'gmd', gmd);
	else
		X = X(:, col);
		idx = kmeans(X, ncomp, 'emptyaction', 'singleton');
		if size(X, 1) <= size(X, 2)
			t = ceil(size(X,2)/size(X,1));
			X = repmat(X, t+1, 1);
			idx = repmat(idx, t+1, 1);
		end
		%gmd = gmdistribution.fit(X, ncomp, 'Start', idx, 'SharedCov', true, 'Regularize', reg);
		gmd = gmdistribution.fit(X, ncomp, 'Start', idx, 'SharedCov', false, 'CovType', 'diagonal', 'Regularize', reg);
		gmm = struct('col', col, 'gmd', gmd);
	end
end

function [bestReg] = cvRegularize(grpYX, ys)
	P_REG = [1e-9 1e-7 1e-5];
	nfold = min(5, min(arrayfun(@(yy) size(grpYX{yy+1},1), ys)));
	fid = [];
	for y = ys
		x = grpYX{y+1};
		%grpYX{y+1} = x(randperm(size(x,1)), :);
		p = repmat(1:nfold, ceil(size(x,1)/nfold));
		fid{y+1} = p(1:size(x,1));
	end
	best = 0;
	bestReg = -1;
	for reg = P_REG
		good = 0;
		for i = 1:nfold
			tr = [];
			for y = ys
				tr{y+1} = grpYX{y+1}(fid{y+1}~=i, :);
			end
			mdl = gmmFit(tr, ys, reg);
			xt = [];
			yt = [];
			for y = ys
				xt = [xt; grpYX{y+1}(fid{y+1}==i, :)];
				yt = [yt; repmat(y, sum(fid{y+1}==i), 1)];
			end
			p = gmmPredict(mdl, xt);
			[m, id] = max(p, [], 2);
			good = good + sum(reshape(ys(id),size(yt,1),1) == yt);
		end
		if bestReg < 0 | good > best
			best = good;
			bestReg = reg;
		end
	end
end

