#include<assert.h>
#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<time.h>

#include"linear.h"
/* train.c will define the following:
	struct parameter param;		// set by parse_command_line
	struct problem prob;		// set by read_problem
	struct model *model_;
*/
#define main never_be_called
#include"train.c"
#undef main

typedef double LabelType;	// for liblinear-1.9+

int K = 3;
char *model_path;

// set by readBits
int N, L;
char **bit;

void readBits(char *path) {
	char tmp[1024];
	FILE *fp = fopen(path, "r");
	assert(fp != NULL);
	fgets(tmp, 1023, fp);
	sscanf(tmp, "%d %d", &N, &L);
	bit = (char**)malloc(sizeof(char*)*prob.l);
	for(int i = 0; i < prob.l; ++i) {
		bit[i] = (char*)malloc(sizeof(char)*(L+3));
		fgets(bit[i], L+2, fp);
	}
}

void parseArgs(int argc, char *argv[]) {
	int n = argc;
	char datafile[4096], modelfile[4096], *bit_path = NULL;
	for(int i = 1; i < argc; ++i) {
		if(strcmp(argv[i], "-K") == 0 && i+1 < argc) {
			++i;
			sscanf(argv[i], "%d", &K);
			n -= 2;
		}
		else if(strcmp(argv[i], "-data") == 0 && i+1 < argc) {
			++i;
			strcpy(datafile, argv[i]);
			strcat(datafile, ".svm");
			--n;
		}
		else if(strcmp(argv[i], "-model") == 0 && i+1 < argc) {
			++i;
			model_path = argv[i];
			--n;
		}
		else if(strcmp(argv[i], "-bit") == 0 && i+1 < argc) {
			++i;
			bit_path = argv[i];
			n -= 2;
		}
	}
	assert(K > 0);
	assert(bit_path != NULL);
	int did = -1, mid = -1, i, j;
	char **args = (char**)malloc(sizeof(char*)*(n+1));
	for(i = j = 0; i < argc; ++i) {
		if(strcmp(argv[i], "-K") == 0 && i+1 < argc) ++i;
		else if(strcmp(argv[i], "-data") == 0 && i+1 < argc) did = ++i;
		else if(strcmp(argv[i], "-model") == 0 && i+1 < argc) mid = ++i;
		else if(strcmp(argv[i], "-bit") == 0 && i+1 < argc) ++i;
		else args[j++] = argv[i];
	}
	if(did != -1) args[j++] = datafile;
	if(mid != -1) args[j++] = model_path;

	parse_command_line(n, args, datafile, modelfile);
	param.solver_type = L2R_LR;
	param.eps = 1e-5;
	bias = 1;
	read_problem(datafile);
	const char *error_msg = check_parameter(&prob,&param);
	if(error_msg)
	{
		fprintf(stderr,"Error: %s\n",error_msg);
		exit(1);
	}
	free(args);
	readBits(bit_path);
}

inline char* getModelFileName(char *str) {
	sprintf(str, "%s/model.mdl", model_path);
	return str;
}

inline char* getBaseModelFileName(char *str, int id) {
	sprintf(str, "%s/base%03d.mdl", model_path, id);
	return str;
}

inline double doCV(double c, LabelType *tgt) {
	int i, s;
	param.C = c;
	cross_validation(&prob, &param, 5, tgt);
	for(s = i = 0; i < prob.l; ++i)
		if(tgt[i] == prob.y[i]) ++s;
	return s;
}

double cs[9] = {0.5, 2, 0.125, 8, 0.03125, 32, 0.0078125, 128, 512};
double selectParamCV() {
	LabelType *tgt = (LabelType*)malloc(sizeof(LabelType)*prob.l);
	double c = param.C;
	double acc = doCV(c, tgt), m;
	int i;
	for(i = 0; i < 9; ++i) {
		m = doCV(cs[i], tgt);
		if(m > acc) acc = m, c = cs[i];
	}
	param.C = c;
	return acc / prob.l;
}

int main(int argc, char *argv[]) {
	parseArgs(argc, argv);

	int i, j, k, m;
	int *seg = (int*)malloc(sizeof(int)*((L+K-1)/K+1));
	for(i = j = 0; j < L; ++i, j += K) seg[i] = j;
	m = i;
	seg[m] = L;
	if(L%K == 1) seg[--m] = L;

	char tmp[4096], *p;
	FILE *fmdl = fopen(getModelFileName(tmp), "w");
	assert(fmdl != NULL);
	fprintf(fmdl, "%d %d\n", N, L);
	fprintf(fmdl, "%d\n", m);
	for(i = 0; seg[i] < L; ++i) {
		fprintf(fmdl, "%s %d", getBaseModelFileName(tmp, i), seg[i+1]-seg[i]);
		for(j = seg[i]; j < seg[i+1]; ++j)
			fprintf(fmdl, " %d", j);
		fputc('\n', fmdl);
	}
	fclose(fmdl);

	struct problem prob0;
	prob0.n = prob.n;
	prob0.l = prob.l;
	prob0.bias = prob.bias;
	prob0.x = (struct feature_node**)malloc(sizeof(struct feature_node*)*prob.l);
	prob0.y = (LabelType*)malloc(sizeof(LabelType)*prob.l);
	memcpy(prob0.x, prob.x, sizeof(struct feature_node*)*prob.l);
	memcpy(prob0.y, prob.y, sizeof(LabelType)*prob.l);
	for(i = 0; seg[i] < L; ++i) {
		prob.l = 0;
		for(j = 0; j < prob0.l; ++j) {
			char valid = 1;
			for(m = 0, p = bit[j], k = seg[i]; k < seg[i+1] && valid; ++k) {
				if(p[k] == '-') valid = 0;
				m = (m<<1) | (p[k]-'0');
			}
			if(valid) {
				assert(0 <= m && m < (1<<(seg[i+1]-seg[i])));
				prob.x[prob.l] = prob0.x[j];
				prob.y[prob.l] = m;
				++prob.l;
			}
		}
		double t = selectParamCV();
		fprintf(stderr, "seg[%d] (%d - %d) param c = %lg, acc = %lg\n",
				i, seg[i], seg[i+1], param.C, t);
		model_ = train(&prob, &param);
		save_model(getBaseModelFileName(tmp, i), model_);
		free_and_destroy_model(&model_);
	}
}


