#!/usr/bin/env python

import random
import subprocess
import sys
random.seed()

DRY_RUN = False
REALTIME = True

MATLAB = None
SCRIPT_NAME = None
SCRIPT_FP = None

def _getPipe():
	global SCRIPT_FP, DRY_RUN, MATLAB
	if DRY_RUN: return sys.stdout
	else: return MATLAB.stdin if REALTIME else SCRIPT_FP

def init(dry=False, realtime=True):
	global DRY_RUN, SCRIPT_FP, SCRIPT_NAME, MATLAB
	DRY_RUN = dry
	REALTIME = realtime
	if DRY_RUN: return
	if REALTIME and MATLAB is None:
		MATLAB = subprocess.Popen(['matlab'], stdin=subprocess.PIPE, stdout=sys.stdout, stderr=sys.stderr)
	elif not REALTIME and SCRIPT_FP is None:
		SCRIPT_NAME = '/tmp/matlabcomm-' + '%06x'%random.randint(0, 10000000)
		print >> sys.stderr, "temp matlab script", SCRIPT_NAME
		SCRIPT_FP = open(SCRIPT_NAME, 'w')

def assign2DMatrix(name, val):
	p = _getPipe()
	p.write(name + '=[')
	for r in val:
		p.write(' ...\n'.join(map(str,r)) + ';\n')
	p.write('];\n')
	p.flush()

def assignVector(name, val):
	p = _getPipe()
        ss = map(str, val)
        p.write(name + '=[' + '; ...\n'.join([ ';'.join(ss[i:i+30]) for i in xrange(0,len(ss),30) ]) + '];\n')
	p.flush()

def execute(s):
	p = _getPipe()
	p.write(s+'\n')
	p.flush()

def close():
	global SCRIPT_NAME, SCRIPT_FP, MATLAB, DRY_RUN
	if DRY_RUN: return
	if REALTIME:
		if MATLAB.returncode is not None:
			raise Exception('premature end of Matlab')
		MATLAB.stdin.close()
	else:
		SCRIPT_FP.close()
		MATLAB = subprocess.Popen(['matlab'], stdin=open(SCRIPT_NAME), stdout=sys.stdout, stderr=sys.stderr)
	MATLAB.wait()
