#!/usr/bin/env python2.7

import argparse
import json
import os

import evaluate
import util

def checkNLabel(labelfile):
	arr = [ len(s.rstrip()) for s in open(labelfile) ]
	assert min(arr) == max(arr), "different number of labels"
	return arr[0]

def genCodeModel(codemodel, code, n, l, k):
	codergen = "./coder/" + code + "-gen"
	seed = util.randString(7, util.DIGIT+"abcdef")
	util.call([codergen, "-n", str(n), "-l", str(l), "-k", str(k), "-model", codemodel, "-seed", seed])

def encode(codeword, code, codemodel, labels):
	coderenc = "./coder/" + code + "-enc"
	util.call([coderenc, "-model", codemodel, "-label", labels, "-bits", codeword])

def train(args):
	args.mdldir = os.path.normpath(args.mdldir) + os.sep
	if not os.path.exists(args.mdldir):
		os.makedirs(args.mdldir)
	assert os.path.isdir(args.mdldir), "model-dir is not directory"

	json.dump({'code':args.code, 'learner':args.learner}, open(args.mdldir+'basic', 'w'))
	nlabel = checkNLabel(args.labels)
	genCodeModel(args.mdldir+'cdmdl', args.code, nlabel, args.length, args.base)
	encode(args.mdldir+'train_cdwrd', args.code, args.mdldir+'cdmdl', args.labels)
	trainer = "./learner/kps-" + args.learner + "-train"
	util.call([trainer, "-data", args.data, "-bit", args.mdldir+'train_cdwrd', "-model", args.mdldir, "-K", str(args.base)])

def test(args):
	args.mdldir = os.path.normpath(args.mdldir) + os.sep
	js = json.load(open(args.mdldir+'basic'))
	code, learner = js['code'], js['learner']
	dec, meta = args.decoder.split('-')
	predictor = "./learner/kps-" + learner + "-predict"
	metapredictor = "./meta/kps-" + meta
	coderdec = "./coder/" + code + "-dec-" + dec
	util.call([predictor, "-data", args.data, "-model", args.mdldir+"model.mdl", "-bit", args.mdldir+'test_raw.pdt'])
	util.call([metapredictor, "-raw", args.mdldir+'test_raw.pdt', "-codeword", args.mdldir+'test_cdwrd.pdt'])
	util.call([coderdec, "-model", args.mdldir+'cdmdl', "-bits", args.mdldir+'test_cdwrd.pdt', "-label", args.labels])
	if args.truth is not None:
		encode(args.mdldir+'test_cdwrd.truth', code, args.mdldir+'cdmdl', args.truth)
		r = evaluate.evalOne(args.truth, args.mdldir+'test_cdwrd.truth', args.mdldir+'test_cdwrd.pdt', args.labels)
		for (k,n) in [('acc','Subset Accuracy'), ('ham','Hamming Loss'), ('microf1','Micro-F1'), ('macrof1','Macro-F1')]:
			print n, r[k]

if __name__ == "__main__":
	parser = argparse.ArgumentParser(description='Multi-label classification using error-correcting codes.')
	subparsers = parser.add_subparsers(help='sub-commands', dest='action')
	tr_parser = subparsers.add_parser('train', help='train model')
	tr_parser.add_argument('--code', type=str, choices=['rrep','hamr','bch','ldpc','clr','none'], help='error-correcting code', default='hamr')
	tr_parser.add_argument('--length', type=int, help='codeword length', default=127)
	tr_parser.add_argument('--base', type=int, help='base multi-label learner (1 for BR, k>1 for k-powerset)', default=1)
	tr_parser.add_argument('--learner', type=str, choices=['svm','rf','linear','gmm'], help='base multi-class/binary learner', default='linear')
	tr_parser.add_argument('data', type=str, metavar='DATA', help='training data')
	tr_parser.add_argument('labels', type=str, metavar='LABELS', help='training labels')
	tr_parser.add_argument('mdldir', type=str, metavar='MODEL-DIR', help='saving model in this directory')
	ts_parser = subparsers.add_parser('test', help='test model')
	ts_parser.add_argument('--decoder', type=str, choices=['alg-hard','geo-hard','geo-soft'], help='ECC decoder', default='geo-soft')
	ts_parser.add_argument('--truth', type=str, help='ground truth labels')
	ts_parser.add_argument('data', type=str, metavar='DATA', help='testing data')
	ts_parser.add_argument('mdldir', type=str, metavar='MODEL-DIR', help='loading model from this directory')
	ts_parser.add_argument('labels', type=str, metavar='LABELS-OUTPUT', help='training labels')
	args = parser.parse_args()

	if args.action == 'train':
		train(args)
	else:
		test(args)

